clear all
set mem 32m
set more off

local 0 , `0'

syntax [, irt mimic dif start ]

use nhanes-adl-irt

sum wtint
local meanw = r(mean)

drop pfq061a pfq061k pfq061t
recode pfq* (5 =.a) (7=.b) (9=.c)

egen _pfqmiss = rowmiss( pfq* )
drop if _pfqmiss > 10

local k=0
foreach x of varlist pfq061* {
   local ++k
   gen byte bpfq`k' = (`x'>1) if !missing( `x' )
   local pfqlab `pfqlab' `k' "`x'"
   ren `x' pfq`k'
}

if "`start'" == "start" {
// initial estimates of the loadings
confa (health: pfq*) [pw=wtint2], from(iv) search(off) vce( sbentler )
predict health, reg

  estimates store cfa
  estimates save cfa, replace

  mat cfa = e(b)

  hist health [fw=int(wtint2)]
}

reshape long bpfq pfq, i(seqn) j(item)

keep if pfq < 5

gen wqt1 = 1
gen wgt2 = wtint2yr/`meanw'

lab def pfqlab `pfqlab'
lab val item pfqlab

tab item, gen( q )

unab allbutfirst : q2-q17
unab allitems : q1-q17

eq item1 : `allbutfirst'
eq items : `allitems'

if "`start'" == "start" {

// starting values
probit bpfq `allbutfirst'
mat oprob = e(b)

// set up the matrix of starting values
mat myoprob = J(1,1,.)
foreach x of varlist `allitems' {
   mat myoprob = (myoprob, _b[bpfq:`x'])
   local collabs `collabs' bpfq:`x'
}

* foreach x of varlist `allbutfirst' {
forvalues j = 2/17 {
   local x : word `j' of `items'
   mat myoprob = (myoprob, cfa[1,colnumb(cfa, "lambda_`j'_1:_cons")])
   local collabs `collabs' seq1_1l:`x'
}

mat myoprob = (myoprob, cfa[1,colnumb(cfa, "phi_1_1:_cons")])
local collabs `collabs' seq1_1:q1

mat myoprob = myoprob[1,2...]

mat colnames myoprob = `collabs'
mat rownames myoprob = y1

mat li myoprob

// start small
gllamm bpfq if seqn < 4200 , ///
   fam( bin ) link( oprobit ) thresh( item1 ) ///
   i( seqn ) eq( items ) weight( wgt ) ///
   trace dots iter(3) from( myoprob ) copy

est store from1
mat from1 = e(b)

mata : from1 = st_matrix("from1")
mata : mata matsave from1 from1, replace

file open labs using from1labs.txt, write text replace
file write labs `"`: colfullnames from1'"' _n
file close labs

}

if "`irt'" == "irt" {

mata : mata matuse from1, replace
mata : st_matrix("from1",from1)
file open labs using from1labs.txt, read text
file read labs flab
file close labs
mat colnames from1 = `flab'

gllamm bpfq q*, ///
   fam( bin ) link( probit ) i( seqn ) eq( items ) weight( wgt ) ///
   trace from( from1 ) copy nocons
est store irt
mat irt = e(b)

// predictions and graphs

}

if "`mimic'" == "mimic" {

mata : mata matuse from1, replace
mata : st_matrix("frommimic",from1)
file open labs using from1labs.txt, read text
file read labs fmlab
file close labs

// MIMIC
mkspline age1 40 age2 60 age3 70 age4 = ridag
eq a1e : age1 age2 age3 age4 riagendr

di `"{inp}`fmlab'"'
foreach x of varlist age1-age4 riagendr {
   mat frommimic = (frommimic, 0)
   local fmlab `fmlab' a1e:`x'
}
di `"{inp}`fmlab'"'
mat colnames frommimic = `fmlab'

gllamm bpfq q*, ///
   fam( bin ) link( probit ) i( seqn ) eq( items ) weight( wgt ) geq( a1e ) ///
   trace from( frommimic ) skip nocons difficult
est store mimic
est save mimic, replace
mat mimic = e(b)

}

if "`dif'" == "dif" {
mata : mata matuse from1, replace
mata : st_matrix("fromdif",from1)
file open labs using from1labs.txt, read text
file read labs fdlab
file close labs

// DIF
gen byte female = riagendr-1
cap noi xi, noomit: i.female*i.item
eq fitems  : `allitems' _IfemXite_1_*

unab finter : _IfemXite_1_*
foreach x of varlist `finter' {
   mat fromdif = (fromdif, 0, 0)
   local fdlab `fdlab' bpfq:`x' seq1_1l:`x'
}
mat colnames fromdif = `fdlab'

gllamm bpfq q* _IfemXite_1_* , ///
   fam( bin ) link( probit ) i( seqn ) eq( fitems ) weight( wgt ) ///
   trace from( fromdif ) skip nocons
est store dif
est save dif, replace
mat dif = e(b)

local totestload 0
local totestdiff 0
forvalues k=1/`: word count `coefs'' {
   if strpos( "`: word `k' of `coefs' '", "fem") {
      if strpos( "`: word `k' of `coefs' '", "bpfq") {
         local totestdiff `totestdiff'=_b[`: word `k' of `coefs'']
      }
      else {
         local totestload `totestload'=_b[`: word `k' of `coefs'']
      }
   }
}

test `totestdiff'
test `totestload'

}

exit
