clear all

set more off

cap log close
log using nhanes-gmm-sem, replace

do nhanes-data

// keep only the best indicators
drop pfd* bpfq*
keep seqn  pfq1 pfq5 pfq8 pfq11 pfq15 ridstatr- female

do nhanes-sem-def-reduced

// explanatory variables
gen float agedyr  = ridageyr/10
lab var agedyr "Age, deca-years"
local exovar agedyr female bmxbmi highbp

// indicators
unab endovar : pfq*

// estimation sample
gen byte _touse = 1
markout _touse `exovar' `endovar'
drop if ~_touse

// define parameter names:
   // the first 4 are lambda2..lambda5
   forvalues k=2/`: word count `endovar'' {
      local parlist `parlist' lambda`k'
   }

   // the next 10 are exogenous covariances
   forvalues i=1/`: word count `exovar'' {
      forvalues j=`i'/`: word count `exovar'' {
         local parlist `parlist' phi_`i'_`j'
      }
   }

   // the next one is VarZeta
   local parlist `parlist' Vzeta

   // the next 4 are regression coefficients
   forvalues i=1/`: word count `exovar'' {
      local parlist `parlist' gamma`i'
   }

   // the next 5 are measurement errors
   forvalues k=1/`: word count `endovar'' {
      local parlist `parlist' psi`k'
   }

local gmmsemopt parameters(`parlist') nequations( 45 ) obsvar(`exovar' `endovar') conv_maxiter(200)

// stupid starting values
matrix from1 = J(1,`: word count `parlist'',1)


// ULS
gmm sem4gmm if _touse , `gmmsemopt' winitial( identity ) from( from1 ) onestep vce( unadj )
est store r_uls_homosked
est save  r_uls_homosked, replace
// save the resulting estimates for future use as starting values
mat bULS = e(b)

gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bULS ) onestep vce( robust )
est store r_uls_heterosked
est save  r_uls_heterosked, replace

// diagonally weighted least squares
gmm sem4gmm , `gmmsemopt' winitial( unadjusted, independent ) from(bULS) onestep vce( unadj )
est store r_dwls_1step_homosked
est save  r_dwls_1step_homosked, replace
mat bDWLS = e(b)

gmm sem4gmm , `gmmsemopt' winitial( unadjusted, independent ) from(bDWLS) onestep vce( robust )
est store r_dwls_1step_heterosked
est save  r_dwls_1step_heterosked, replace

gmm sem4gmm , `gmmsemopt' winit(unadj, indep) wmat(unadj, indep) from(bDWLS) twostep vce(unadj)
est store r_dwls_2step_homosked
est save  r_dwls_2step_homosked, replace

gmm sem4gmm , `gmmsemopt' winit(unadj, indep) wmat(unadj, indep) from(bDWLS) twostep vce(robust)
est store r_dwls_2step_heterosked
est save  r_dwls_2step_heterosked, replace

// efficient GMM
gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bDWLS ) twostep vce( unadj )
est store r_effls_2step_homosked
est save  r_effls_2step_homosked, replace
mat bADF = e(b)

gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bADF ) twostep vce( robust )
est store r_effls_2step_heterosked
est save  r_effls_2step_heterosked, replace

gmm sem4gmm , `gmmsemopt' winit( id ) wmat( unadj ) from( bADF ) igmm igmmiter(30) vce( unadj )
est store r_effls_igmm_homosked
est save  r_effls_igmm_homosked, replace

gmm sem4gmm , `gmmsemopt' winit( id ) wmat( robust) from( bADF ) igmm igmmiter(30) vce( robust )
est store r_effls_igmm_heterosked
est save  r_effls_igmm_heterosked, replace

// compare results
est tab r_*_homosked, se
est tab r_*_heterosked, se
est tab r_eff*, se stats(J)

log close
view nhanes-gmm-sem.smcl

exit
