clear all

set more off

do nhanes-data
do nhanes-sem-def

// explanatory variables
local exovar age1 age2 age3 age4 female bmxbmi highbp

// indicators
unab endovar : pfq*

// estimation sample
gen byte _touse = 1
markout _touse `exovar' `endovar'
drop if ~_touse

// define parameter names:
   // the first 16 are lambda2..lambda17
   forvalues k=2/`: word count `endovar'' {
      local parlist `parlist' lambda`k'
   }

   // the next 28 are exogenous covariances
   forvalues i=1/`: word count `exovar'' {
      forvalues j=`i'/`: word count `exovar'' {
	     local parlist `parlist' phi_`i'_`j'
	  }
   }
   
   // the next one is VarZeta
   local parlist `parlist' Vzeta
   
   // the next 7 are regression coefficients
   forvalues i=1/`: word count `exovar'' {
      local parlist `parlist' gamma`i'
   }
   
   // the next 17 are measurement errors
   forvalues k=1/`: word count `endovar'' {
      local parlist `parlist' psi`k'
   }

local gmmsemopt parameters(`parlist') nequations( 300 ) obsvar(`exovar' `endovar')

// stupid starting values
matrix from1 = J(1,`: word count `parlist'',1)


// ULS
gmm sem4gmm if _touse , `gmmsemopt' winitial( identity ) from( from1 ) onestep vce( unadj )
est store uls_homosked
est save  uls_homosked
// save the resulting estimates for future use as starting values
mat bULS = e(b)

gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bULS ) onestep vce( robust )
est store uls_heterosked

// diagonally weighted least squares
gmm sem4gmm , `gmmsemopt' winitial( unadjusted, independent ) from(bULS) onestep vce( unadj )
est store dwls_1step_homosked
mat bDWLS = e(b)
gmm sem4gmm , `gmmsemopt' winitial( unadjusted, independent ) from(bDWLS) onestep vce( robust )
est store dwls_1step_heterosked
gmm sem4gmm , `gmmsemopt' winit(unadj, indep) wmat(unadj, indep) from(bDWLS) twostep vce(unadj)
est store dwls_2step_homosked
gmm sem4gmm , `gmmsemopt' winit(unadj, indep) wmat(unadj, indep) from(bDWLS) twostep vce(robust)
est store dwls_2step_heterosked

// efficient GMM
gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bDWLS ) twostep vce( unadj )
est store effls_2step_homosked
gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bULS ) twostep vce( robust )
est store effls_2step_heterosked
gmm sem4gmm , `gmmsemopt' winit( id ) wmat( unadj ) from( bULS ) igmm vce( unadj )
est store effls_igmm_homosked
gmm sem4gmm , `gmmsemopt' winit( id ) wmat( robust) from( bULS ) igmm vce( robust )
est store effls_igmm_heterosked

exit
